import scriptconfig as scfg


class LearnConfig(scfg.Config):
    """
    Default configuration for Learn task

    Example:
        >>> from learn.protocol.learn_config import LearnConfig
        >>> config = LearnConfig()
        >>> print('config = {!r}'.format(config))
    """
    default = {
        'dataset_dir': scfg.Path("/mnt/b8ca6451-1728-40f1-b62f-b9e07d00d3ff/data/lwll_datasets",
                                 help='location of dataset (usually called lwll_datasets'),

        'tasks_to_run': scfg.Value(['problem_test_image_classification'],
                                   help='Array or Tuple with the tasks to Run'),

        'domain_network_selection_algo': scfg.Path(
            "templateDomainNetworkSelection/domainNetworkSelection.py",
            help='Algorithm to select the source domain and network for the task'),

        'algoSelection_algo': scfg.Path(
            "algoSelection.py",
            help='Algorithm for Selection of algorithms for query and classifications'),

        'query_algo': scfg.Path(
            'templateQueryAlgo/query_algo.py',
            help='path from algorithm folder to the query algorithm '
                 '(Use "auto" to have the algorithm select'),

        'image_classification_algo': scfg.Path(
            'CloserLookFewShot/image_classification.py',
            help='path from algorithm folder to the image classification algorithm'
                 '(Use "auto" to have the algorithm select'),

        'object_detection_algo': scfg.Path(
            'templateObjectDetection/object_detection.py',
            help='path from algorithm folder to the object detection algorithm'
                 '(Use "auto" to have the algorithm select'),

        'cuda': scfg.Value(True),
    }




